///<ObjektNummer>2325</ObjektNummer>

#if !defined(h_Fat)
#define h_Fat

// Defines
#define F_CPU 20000000
#define MCU ATMEGA644P

#include <avr\io.h>
#include "MmcCard.h"

#include "M8_Mx8_SPI.h"

#include <string.h>

extern class Fat* globCurrentFat;

#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

//Volume Boot Record location in Master Boot Record
#define VBR_ADDR 			0x1C6

//Block Size in Bytes
#define BlockSize			512

//Master Boot Record
#define MASTER_BOOT_RECORD	0

//define ASCII
#define SPACE 				0x20
#define DIR_ENTRY_IS_FREE   0xE5
#define FIRST_LONG_ENTRY	0x01
#define SECOND_LONG_ENTRY	0x42

//define DIR_Attr
#define ATTR_LONG_NAME		0x0F
#define ATTR_READ_ONLY		0x01
#define ATTR_HIDDEN			0x02
#define ATTR_SYSTEM			0x04
#define ATTR_VOLUME_ID		0x08
#define ATTR_DIRECTORY		0x10
#define ATTR_ARCHIVE		0x20

// FileCreate-Standard-Attribute
#define FILECREATE_TIME		0
#define FILECREATE_DATE		0
#define FILECREATE_ATTR		ATTR_ARCHIVE

struct BootSec 
{
	uint8_t		BS_jmpBoot[3];
	uint8_t		BS_OEMName[8];
	uint16_t	BPB_BytesPerSec; 	//2 bytes	// Anzahl Bytes je Sector
	uint8_t		BPB_SecPerClus;					// Anzahl sektoren je Cluster
	uint16_t	BPB_RsvdSecCnt; 	//2 bytes	// Anzahl reservierter Sectoren 
	uint8_t		BPB_NumFATs;					// Anzahl der Kopieen der FAT
	uint16_t	BPB_RootEntCnt; 	//2 bytes	// Anzahl der mglichen Eintrge im Hauptverzeichnis
	uint16_t	BPB_TotSec16; 		//2 bytes	// Anzahl Sektoren in kleineren Partitionen
	uint8_t		BPB_Media;						// Medientyp-Kennzeichner, z.B: F0=Diskette, F8=HDD, 
	uint16_t	BPB_FATSz16; 		//2 bytes	// Sectoren je FAT
	uint16_t	BPB_SecPerTrk; 		//2 bytes	// Sectoren jen Track
	uint16_t	BPB_NumHeads; 		//2 bytes	// Anzahl der Heads
	uint32_t	BPB_HiddSec; 		//4 bytes	// reservierte Sectoren am Begin des Datentrgers
	uint32_t	BPB_TotSec32; 		//4 bytes	// Gesamtzahl der Sektoren in der Partition
};

struct FatData
{
	uint8_t secPerCluster;
	uint16_t fatOffset;
	uint16_t clusterOffset;
	uint16_t secPerFat;
	uint16_t firstRootDirSec;
	uint16_t volumeBootRecordAddr;
};

#define String StringBig
/////// DEBUG /////////////
//#define IsDEBUG	// auch im Paket "Debug" schalten
	#define DEBUG_BAUD 500000
	#define DEBUG_UART_NR 0
	#include "debugUart.h"

class Fat : public MmcCard
{

public:
	//	extractNextCluster()
	//	Routine zum extrahieren des Folge-Clustereintrags aus einer FatTabelle anhand der akt. Clusternummer
	//	
	//	PE:
	//		aktCluster = Cluster fr den der FolgeCluster gesucht wird fat.FatOffset
	//		
	//	PA:
	//		0xffff wenn Fehler oder nichts folgt
	uint16_t extractNextCluster(uint16_t aktCluster);
	//	absSecFromCluster()
	//	Berechnet aus der Clusternummer den zugehrigen 1. Sektor
	uint32_t absSecFromCluster(uint16_t cluster);

	//	initFat()
	uint8_t initFat();
	//	clusterDataStore()
	//	Auslesen Cluster Size der MMC/SD Karte und Speichern der gre ins RAM
	//	Auslesen Cluster Offset der MMC/SD Karte und Speichern der gre ins RAM
	uint8_t clusterDataStore();
	//	addr()
	//	Auslesen der Adresse des Volume Boot Record von MBR
	uint16_t addr();
	//	findInit()
	//	Initialisierungsroutine fr fat_find_next(), wenn dir_cluster = 0 suche im RootDir
	void findInit(uint16_t dirCluster, class DirEntryCache *directory);
	//	findNext()
	//	holt nchsten gltigen eintrag aus dem directory
	//	Einschrnkung: es wird immer nur max. der Erste Cluster gelesen
	//	
	//	PE:
	//		directory = zu verwendende DirEntryCache-Struktur
	//		    getFreeEntry = wenn true wird nach freien Eintrgen gesucht
	//		
	//	PA:
	//		false = fehler/nicht gefunden
	bool findNext(class DirEntryCache *directory, bool getFreeEntry=false);
	//	writeBackBuffer()
	//	Rckschreiben des Buffer wenn notwendig
	void writeBackBuffer();
	//	setNextCluster()
	//	Trgt Cluster in FAT ein ordnet einem Cluster 'old' einen cluster 'new' zu und beendet die Datei im Cluster 'new' mit 0xffff
	bool setNextCluster(uint16_t oldCluster, uint16_t newCluster);
	//	setClusterFatEntry()
	//	setzt einen FAT-ClusterEintrag
	bool setClusterFatEntry(uint16_t clusterNr, uint16_t entry);
	//	getFreeCluster()
	//	holt nen freien Clusternummer
	//	
	//	
	//	PA:
	//		0 = Error
	uint16_t getFreeCluster();
	//	writeDirEntry()
	//	Schreib DirEntryCache zurck, wenn er gendert wurde
	void writeDirEntry(DirEntryCache* dir);

	//	fileDelete()
	void fileDelete(DirEntryCache* dir);
	FatData fat;
	//	Konstruktor
	Fat();

	//	Destruktor
	~Fat();

private:

protected:

};

#endif
