//----------------------------------------------------------------------
// Titel     : myEthernet Sample02
//----------------------------------------------------------------------
// Prozessor : ATmega644P
// Takt		 : 20 MHz
// Sprache   : C
// Datum     : ...
// Version   : ...
// Autor     : ...
// LinkerOptions: myEthernet_Debug.a
//----------------------------------------------------------------------
// Es zeigt eine minimale Version eines Webservers, es werden hierbei nur die ntigsten
// Funktionen realisiert.
// analog Sample01 zustzlich:
// 	- Debug-Infos per UART 500000 Baud
//  - Systemzeit-Ausgabe
// Die Nutzung der Quelltexte und Bibliotheken erfolgt auf eigene Gefahr und 
// kann auch selbstndig verndert werden (z. B. fr eigene Projekte). 
// Wir bernehmen keine Haftung fr die Software.
//----------------------------------------------------------------------
#define 	F_CPU 20000000	// Taktferquenz
#include	<avr\io.h>			// AVR Register und Konstantendefinitionen
#define		IsDEBUG					// aktiviert DebugUart mit 500000 Baud
//----------------------------------------------------------------------
#include "myEthernet.h"
//----------------------------------------------------------------------
//=========================================================================================
// Class MyWebserver
// - Diese Klasse behandelt die GET-Anfragen und sendet den Webseiten-Inhalt
//=========================================================================================
class MyWebserver : public WebserverHardware
{
	public:
	bool config();
	protected:
	//	onWebRequestGet()
	//	PE:
	//		socket = akteuller Socket
	//		pFile = Zeiger auf URL, mit 0x00 abgeschlossen
	//		pPara = Zeiger auf URL-Parameter-Start, mit 0x00 getrennt
	//		pParaEnd = Zeiger auf Ende der Parameter, zeigt auf 0x00
	virtual void onWebRequestGet(uint16_t socket, unsigned char * pFile, unsigned char * pPara, unsigned char * pParaEnd);
};
//======================================================================
// Main + globale Instanzen + Interrupts + Fuses + Lockbits
//======================================================================
DebugUart debug;
MyWebserver webserver;
Timer1 timer1;
uint32_t systime;
void init();
//======================================================================
main ()						// Hauptprogramm, startet bei Power ON und Reset
{
	// Initialisierung
	init();	
	// Mainloop	
	while(true)
	{
		webserver.work();
	}
}

ISR(TIMER1_COMPA_vect)
{
	// Systemzeit
	static uint8_t count=100;	// zhlt die 1/100 Sekunden
	count--;
	if(count==0)	// wenn Sekunde um ist
	{
		systime++;
		count=100;
	}

	// Ereignisbehandlung/Polling
	webserver.workOnTimer10ms();
}

__fuse_t __fuse __attribute__((section (".fuse"))) =
{
	/*low*/ 0xFF,
	/*high*/ 0xD9,
	/*extended*/ 0xFD
};

unsigned char __lock __attribute__((section (".lock"))) = {0xFC};
//======================================================================
void init()
{
	systime = 0;
	DebugPrintFlash("--- init start ---\n");
	///////// alles aus ////////////////
	cli();
	timer1.stop();
	webserver.spiStop();
	webserver.reset.on();
	// sd-Card-aus
	sbi(DDRC,6);sbi(PORTC,6);	// SS der SD-Card

	//////// Timer 10ms	
	// 1/100s = 10ms Einstellen
	timer1.config_compare( Timer1::sourcePrescale8, F_CPU/8/100  /* =10000 bei 8MHz; =20000 bei 16MHz */);
	timer1.configInt_compare(true,false);	// incl. sei()

	//////// Netzwerk starten /////////
	webserver.config();

	///// grne LED
	sbi(DDRC,4);	// Ausgang
	// rote LED
	sbi(DDRC,5);	// Ausgang

	// ##### TODO ######################################################
	// ## hier ggf. weitere Initialisierungen durchfhren

	sbi(PORTC,4);	// grne LED an

	DebugPrintFlash("--- init end ---\n");
}

//=========================================================================================
// Class MyWebserver - Implementation
//=========================================================================================
bool MyWebserver::config()
{
	bool ok=true;
	configHardware();
	configEthernet();
	config_webserver(80); 	// Port bergeben
	if(!checkConnection())
		ok=false;

	// ##### TODO ######################################################
	// ## ggf. Einstellungen ndern
	//myIP=IP(192,168,20,96);
	//myNetworkmask=IP(255,255,255,0);
	//setMyMacAddress(0x35,0x85,0x47,0x02,0x01,0x96);
	return ok;
}

// onWebRequestGet -----------------------------------------------------
// verarbeitet die GET-Anfragen an den Webserver
// Eine GET-Anfrage ist auf eine maximale Lnge von 64 Byte beschrnkt.
// Die Antwort kann eine beliebige Gre haben.
//	PE:
//		socket = aktueller Socket
//		pFile = Zeiger auf URL, mit 0x00 abgeschlossen
//		pPara = Zeiger auf URL-Parameter-Start, mit 0x00 getrennt
//		pParaEnd = Zeiger auf Ende der Parameter, zeigt auf 0x00
void MyWebserver::onWebRequestGet(uint16_t socket, unsigned char * pFile, unsigned char * pPara, unsigned char * pParaEnd)
{
	DebugPrintDez("systime=",systime);
	DebugPrintFlash("onWebRequestGet:\n");
	DebugPrintFlash("...pFile=");
	DebugPrintRam(pFile,-1);
	DebugPrintFlash("\n");
	DebugPrintFlash("...pPara=");
	DebugPrintRam(pPara,pParaEnd-pPara);
	DebugPrintFlash("\n");

	// ##### TODO ######################################################
	// ## URL und GET-Paramter verarbeiten, sowie HTML-Seite senden

	bool ok;
	ok=sendHttpHeader(socket,'h');

	static uint8_t html1[] PROGMEM= {
		"<HEAD>" "<TITLE>myEthernet</TITLE>" "</HEAD>" "<BODY> <h1>myAVR - myEthernet</h1>"
		"Gratulation! Der Webserver funktioniert.<br>"
	};	// Beachte: einige WnAVR Kompiler bringen hier eine Warnung, die nicht beachtet werden darf
	ok=putSocketData( socket, sizeof(html1)-1,html1, FLASH );

	String txt;
	txt.format("%d",systime);	
	txt="Er ist schon seit "+txt+" Sekunden online.<br>";
	ok=putSocketData( socket, txt.getSize(),txt.pBuffer, RAM );

	static uint8_t html2[] PROGMEM= {
		"</BODY>"
	};	// Beachte: einige WnAVR Kompiler bringen hier eine Warnung, die nicht beachtet werden darf
	ok=putSocketData( socket, sizeof(html2)-1,html2, FLASH );
}
