//----------------------------------------------------------------------
// Titel     : myEthernet Sample03
//----------------------------------------------------------------------
// Prozessor : ATmega644P
// Takt		 : 20 MHz
// Sprache   : C
// Datum     : ...
// Version   : ...
// Autor     : ...
// LinkerOptions: myEthernet_Debug.a
//----------------------------------------------------------------------
// Einfache Nutzung des myEthernet mittels lib_myEthernet.a
// Es zeigt eine minimale Version eines Webservers, es werden hierbei nur die ntigsten
// Funktionen realisiert.
// analog Sample01 zustzlich:
// 	- Debug-Infos per UART 500000 Baud
//  - Systemzeit-Ausgabe
//  - MicroSD-Card mit FAT-Dateisystem
//  - Rckgabe der angeforderten Datei von MicroSD-Card
// Die Nutzung der Quelltexte und Bibliotheken erfolgt auf eigene Gefahr und 
// kann auch selbstndig verndert werden (z. B. fr eigene Projekte). 
// Wir bernehmen keine Haftung fr die Software.
//----------------------------------------------------------------------
#define 	F_CPU 20000000	// Taktferquenz
#include	<avr\io.h>		// AVR Register und Konstantendefinitionen
#define		IsDEBUG
//----------------------------------------------------------------------
#include "myEthernet.h"
//----------------------------------------------------------------------
//=========================================================================================
// Class MyWebserver
// - Diese Klasse behandelt die GET-Anfragen und sendet den Webseiten-Inhalt
//=========================================================================================
class MyWebserver : public WebserverHardware
{
	public:
	bool config();
	protected:
	//	onWebRequestGet()
	//	PE:
	//		socket = akteuller Socket
	//		pFile = Zeiger auf URL, mit 0x00 abgeschlossen
	//		pPara = Zeiger auf URL-Parameter-Start, mit 0x00 getrennt
	//		pParaEnd = Zeiger auf Ende der Parameter, zeigt auf 0x00
	virtual void onWebRequestGet(uint16_t socket, unsigned char * pFile, unsigned char * pPara, unsigned char * pParaEnd);

	bool sendFile(uint16_t socket, char* filename);
};
//======================================================================
// Main + globale Instanzen + Interrupts + Fuses + Lockbits
//======================================================================
DebugUart debug;
MyWebserver webserver;
Timer1 timer1;
MyCard sdCard;
uint32_t systime;
void init();
//======================================================================
main ()						// Hauptprogramm, startet bei Power ON und Reset
{
	// Initialisierung
	init();	
	// Mainloop	
	while(true)
	{
		webserver.work();
	}
}

ISR(TIMER1_COMPA_vect)
{
	// Systemzeit
	static uint8_t count=100;	// zhlt die 1/100 sekunden
	count--;
	if(count==0)	// wenn sekunde um ist
	{
		systime++;
		count=100;
	}

	// Ereignisbehandlung/Polling
	webserver.workOnTimer10ms();
}

__fuse_t __fuse __attribute__((section (".fuse"))) =
{
	/*low*/ 0xFF,
	/*high*/ 0xD9,
	/*extended*/ 0xFD
};

unsigned char __lock __attribute__((section (".lock"))) = {0xFC};
//======================================================================
void init()
{
	systime = 0;
	bool ok=true;
	DebugPrintFlash("--- init start ---\n");
	///////// alles aus ////////////////
	cli();
	timer1.stop();
	webserver.spiStop();
	webserver.reset.on();
	// sd-Card-aus
	sbi(DDRC,6);sbi(PORTC,6);	// SS der SD-Card

	//////// sdCard-init ///////////////
	ok=sdCard.config();
	if(!ok)
	{
		sbi(DDRC,5);sbi(PORTC,5); 	// LED rot
		DebugPrintFlash("FEHLER: bei MicroSD-Card Initialisierung\n");
	}

	//////// Timer 10ms	
	// 1/100s = 10ms Einstellen
	timer1.config_compare( Timer1::sourcePrescale8, F_CPU/8/100  /* =10000 bei 8MHz; =20000 bei 16MHz */);
	timer1.configInt_compare(true,false);	// incl. sei()

	// sd-Card-aus, SPI-Bus fr Netzwerk freihalten
	sbi(DDRC,6);sbi(PORTC,6);	// SS der SD-Card

	//////// Netzwerk starten /////////
	webserver.config();

	///// grne LED
	sbi(DDRC,4);	// Ausgang
	// rote LED
	sbi(DDRC,5);	// Ausgang

	// ##### TODO ######################################################
	// ## hier ggf. weitere Initialisierungen durchfhren

	sbi(PORTC,4);	// grne LED an

	DebugPrintFlash("--- init end ---\n");
}
//======================================================================

//=========================================================================================
// Class MyWebserver - Implementation
//=========================================================================================
bool MyWebserver::config()
{
	bool ok=true;
	configHardware();
	configEthernet();
	config_webserver(80); 	// Port bergeben
	if(!checkConnection())
		ok=false;
	// ##### TODO ######################################################
	// ## ggf. Einstellungen ndern
	//myIP=IP(192,168,20,96);
	//myNetworkmask=IP(255,255,255,0);
	//setMyMacAddress(0x35,0x85,0x47,0x02,0x01,0x96);
	return ok;
}

// onWebRequestGet -----------------------------------------------------
// verarbeitet die GET-Anfragen an den Webserver
// Eine GET-Anfrage ist auf eine maximale Lnge von 64 Byte beschrnkt.
// Die Antwort kann eine beliebige Gre haben.
//	PE:
//		socket = aktueller Socket
//		pFile = Zeiger auf URL, mit 0x00 abgeschlossen
//		pPara = Zeiger auf URL-Parameter-Start, mit 0x00 getrennt
//		pParaEnd = Zeiger auf Ende der Parameter, zeigt auf 0x00
void MyWebserver::onWebRequestGet(uint16_t socket, unsigned char * pFile, unsigned char * pPara, unsigned char * pParaEnd)
{
	DebugPrintDez("systime=",systime);
	DebugPrintFlash("onWebRequestGet:\n");
	DebugPrintFlash("...pFile=");
	DebugPrintRam(pFile,-1);
	DebugPrintFlash("\n");
	DebugPrintFlash("...pPara=");
	DebugPrintRam(pPara,pParaEnd-pPara);
	DebugPrintFlash("\n");

	// ##### TODO ######################################################
	// ## URL und GET-Paramter verarbeiten, sowie HTML-Seite senden

	//	DebugPrintFlash("MeinWebserver::onWebRequestGet(..) systime:");DebugPrintRam(app.getSystimeString().pBuffer,-1);DebugPrintFlash(", ");DebugPrintRam(pFile,-1);DebugPrintFlash(" ? ");DebugPrintRam(pPara,pParaEnd-pPara);DebugPrintFlash("\n");
	// hier website zwischenspeichern, parameter sammeln und wenn alle blcke da
	// - para verarbeiten
	// - webstite laden+senden
	bool ok=true;

	if(!ok || !sendFile(socket,(char*)pFile))
	{
		///// wenn nicht gefunden, Standardseite senden

		ok=sendHttpHeader(socket,'h');
		static uint8_t html1[] PROGMEM= {
			"<HEAD>" "<TITLE>myEthernet</TITLE>" "</HEAD>" "<BODY> <h1>myAVR - myEthernet</h1>"
			"Ups...<br>Die Datei wurde nicht gefunden, oder es ist keine SD-Card eingelegt.<br>"
		};	// Beachte: einige WnAVR Kompiler bringen hier eine Warnung, die nicht beachtet werden darf
		ok=putSocketData( socket, sizeof(html1)-1,html1, FLASH );

		String txt;
		txt.format("%d",systime);	
		txt="Der Server ist seit "+txt+" Sekunden online.<br>";
		ok=putSocketData( socket, txt.getSize(),txt.pBuffer, RAM );

		static uint8_t html2[] PROGMEM= {
			"</BODY>"
		};	// Beachte: einige WnAVR Kompiler bringen hier eine Warnung, die nicht beachtet werden darf
		ok=putSocketData( socket, sizeof(html2)-1,html2, FLASH );
	}
}

// sendFile----------------------------------------------------------------------
// gibt eine Datei von der SD-Card auf dem angegebenen Socket aus
// PE:
//		socket = aktueller Socket
//		filename = Name der Datei, bei Leersteing (""), wird die Datei 'index.htm' verwendet
bool MyWebserver::sendFile(uint16_t socket, char* filename)
{
	//DebugPrintFlash("MeinWebserver::sendFile(..) ===> ");DebugPrintRam(filename,-1);DebugPrintFlash("\n");
	spiStop();	// da SD-Card bentigt
	bool ok=true;
	Directory dir;
	File file;
	// wenn default
	dir.getEntry().fat->lastError=0;
	if(filename[0]==0)
		ok=dir.fileFind("index.htm");
	else
		ok=dir.fileFind(filename);
	// wenn MMC-Card-Problem -> reinit
	if(!ok && dir.getEntry().fat->lastError)
	{
		ok=sdCard.config();
	}
	if(ok)
		ok=file.open(dir.getEntry());
	if(ok)
	{
		DebugPrintDez("... file gefunden, Lnge=",file.entry.dirEntry.fileSize);
		///// Dateityp
		uint8_t filetyp=0;
		if( file.getName()[8]=='H' && file.getName()[9]=='T' && file.getName()[10]=='M'	)	//HTM
			filetyp='h';
		else if (file.getName()[8]=='G' && file.getName()[9]=='I' && file.getName()[10]=='F')
			filetyp='g';
		else if (file.getName()[8]=='J' && file.getName()[9]=='P' && file.getName()[10]=='G')
			filetyp='j';

		///// HTTP-Header senden
		if(filetyp=='g')	//GIF
			sendHttpHeader(socket,'g');
		else if(filetyp=='j')	//JPG
			sendHttpHeader(socket,'j');
		else
			sendHttpHeader(socket,'h');		//sonst Text (z.B. HTM)

		// senden Blockweise, angepasst an unsere Tcp-Blockgre
		char buf[MAX_TCP_Datalenght];
		uint32_t startTime=systime;
		uint8_t i;
		do
		{
			// bis ein block voll
			for(i=0;i<MAX_TCP_Datalenght;)
			{
				char c=file.readByte();
				if(file.lastError)	// z.B. EOF
					break;
				buf[i++]=c;
			}
			// Datei/Paketinhalt ausgeben
			spiStart();
			ok=putSocketData( socket, i , (uint8_t*)&buf, RAM );
			// wait
			for(uint16_t volatile w=0;w<1000;w++)
			{}
			spiStop();
		}
		while(ok && (!file.isEof() ));	// solange noch daten da sind
		//DebugPrintDez(".. bentigte Zeit in 10ms-Ticks=",systime-startTime);
		file.close();
	}
	else
		DebugPrintFlash("...nicht gefunden\n");	//###DEBUG

	spiStart();	// jetzt wieder Ethernet
	return ok;
}
